import { Button, Dropdown, Flex } from "antd";
import {
  isSideMenuCollapsed,
  setIsSideMenuCollapsed,
} from "../SideMenu/sideMenuSlice";
import { useDispatch, useSelector } from "react-redux";
import {
  EditOutlined,
  EyeOutlined,
  LeftOutlined,
  LogoutOutlined,
  MenuFoldOutlined,
  MenuUnfoldOutlined,
  MoonOutlined,
  SunOutlined,
  UserOutlined,
} from "@ant-design/icons";
import { useNavigate } from "react-router-dom";
import { darkMode, setDarkMode } from "../../reducers/appSlice";
import styles from "./styles.module.css";
import { Constants } from "core/utils/utils";
import webSocketService from "core/services/websocketService";
import { userProfilePictureUrl } from "core/reducers/appSlice";
import MyUserAvatar from "shared/components/MyUserAvatar";
import { useTranslation } from "react-i18next";

type HeaderBarProps = {
  theme?: "light" | "dark";
  onView?: () => void;
  onEdit?: () => void;
  backTo?: string;
  sticky?: boolean;
};

export default function HeaderBar(props: HeaderBarProps = { theme: "light" }) {
  const { t } = useTranslation();
  const dispatch = useDispatch();

  const isCollpased = useSelector(isSideMenuCollapsed);
  const isDarkMode = useSelector(darkMode);
  const profilePictureUrl = useSelector(userProfilePictureUrl);

  const navigate = useNavigate();

  return (
    <Flex
      justify="space-between"
      align="center"
      style={{
        paddingTop: 12,
        paddingLeft: 12,
        paddingRight: 12,
        position: props.sticky ? "sticky" : "relative",
        top: 0,
        zIndex: 999,
      }}
    >
      <Flex align="center">
        <div
          className={isDarkMode ? styles.containerDark : styles.containerLight}
          style={{ borderRadius: 28, padding: 4 }}
        >
          {isCollpased ? (
            <div
              className={styles.iconContainer}
              onClick={() => dispatch(setIsSideMenuCollapsed(false))}
            >
              <MenuUnfoldOutlined className={styles.icon} />
            </div>
          ) : (
            <div
              className={styles.iconContainer}
              onClick={() => dispatch(setIsSideMenuCollapsed(true))}
            >
              <MenuFoldOutlined className={styles.icon} />
            </div>
          )}
        </div>

        {props.backTo && (
          <Button
            type="link"
            style={{ color: isDarkMode ? "#fff" : "#1e1e1e" }}
            onClick={() =>
              navigate(
                props.backTo
                  ? props.backTo
                  : Constants.ROUTE_PATHS.LESSIONS.ROOT
              )
            }
            icon={<LeftOutlined />}
          >
            {t("headerBar.backButton")}
          </Button>
        )}
      </Flex>

      <Flex
        align="center"
        className={isDarkMode ? styles.containerDark : styles.containerLight}
        style={{
          borderRadius: 28,
          paddingLeft: 6,
          paddingRight: 6,
          paddingTop: 4,
          paddingBottom: 4,
        }}
        gap={8}
      >
        {props.onView && (
          <div className={styles.iconContainer} onClick={props.onView}>
            <EyeOutlined className={styles.icon} />
          </div>
        )}

        {props.onEdit && (
          <div className={styles.iconContainer} onClick={props.onEdit}>
            <EditOutlined className={styles.icon} />
          </div>
        )}

        {isDarkMode ? (
          <div
            className={styles.iconContainer}
            onClick={() => dispatch(setDarkMode(false))}
          >
            <SunOutlined className={styles.icon} />
          </div>
        ) : (
          <div
            className={styles.iconContainer}
            onClick={() => dispatch(setDarkMode(true))}
          >
            <MoonOutlined className={styles.icon} />
          </div>
        )}
        <Dropdown
          overlayStyle={{ minWidth: 150 }}
          trigger={["click"]}
          menu={{
            items: [
              {
                key: "1",
                label: t("header.accountSettings"),
                icon: <UserOutlined />,
                onClick: () => navigate(Constants.ROUTE_PATHS.ACCOUNT_SETTINGS),
              },
              {
                key: "2",
                label: t("header.logout"),
                icon: <LogoutOutlined />,
                danger: true,
                onClick: () => {
                  webSocketService.disconnect();
                  window.localStorage.removeItem("session");
                  window.location.href = "/";
                },
              },
            ],
          }}
        >
          <div>
            <MyUserAvatar
              size={34}
              profilePictureUrl={profilePictureUrl ? profilePictureUrl : ""}
            />
          </div>
        </Dropdown>
      </Flex>
    </Flex>
  );

  /* return (
    <Header
      style={{
        position: "sticky",
        top: 0,
        zIndex: 10,
        width: "100%",
        display: "flex",
        alignItems: "center",
        padding: 0,
        background: "#fff",
        justifyContent: "space-between",
      }}
    >
      <Button
        type="text"
        icon={isCollpased ? <MenuUnfoldOutlined /> : <MenuFoldOutlined />}
        onClick={() => 
          dispatch(setIsSideMenuCollapsed(!isCollpased))
        }
        style={{ fontSize: 16, width: 64, height: 64 }}
      />

      <div>
        <Link to={Constants.ROUTE_PATHS.LESSIONS.PAGE_EDITOR}>
          <EditOutlined
            style={{
              fontSize: 20,
              width: 64,
              height: 64,
              justifyContent: "center",
            }}
            onClick={() => dispatch(setEditorActive(!isEditorActive))}
          />
        </Link>

        {isDarkMode ? (
          <SunOutlined
            style={{
              fontSize: 20,
              width: 64,
              height: 64,
              justifyContent: "center",
            }}
            onClick={() => dispatch(setDarkMode(false))}
          />
        ) : (
          <MoonOutlined
            style={{
              fontSize: 20,
              width: 64,
              height: 64,
              justifyContent: "center",
            }}
            onClick={() => {
              dispatch(setDarkMode(true));
            }}
          />
        )}
      </div>
    </Header>
  ); */
}
