import { useNavigate, useParams } from "react-router-dom";
import {
  lessonContents,
  lessonThumbnail,
  setCurrentLessonId,
  setEditorActive,
  setLessonContents,
  setLessonThumbnailTitle,
  setLessonThumbnailUrl,
  setPageEditorLessonState,
} from "./lessonPageEditorSlice";
import { useEffect, useRef } from "react";
import { useDispatch, useSelector } from "react-redux";
import { Card, Flex, Typography } from "antd";
import { Constants } from "core/utils/utils";
import HeaderBar from "core/components/Header";
import Droppable from "./Droppable";
import LessonPreviewCard from "shared/components/MyLessonPreviewCard";
import {
  useGetLessonContentsQuery,
  useGetLessonSettingsQuery,
  useUpdateLessonPreviewTitleMutation,
} from "core/services/lessons";
import MyErrorResult from "shared/components/MyResult";
import styles from "./styles.module.css";
import MyEmpty from "shared/components/MyEmpty";
import {
  addWebSocketReconnectListener,
  removeWebSocketReconnectListener,
} from "core/services/websocketService";
import { useTranslation } from "react-i18next";

const PreviewCard: React.FC = () => {
  const dispatch = useDispatch();
  const { lessonId } = useParams();
  const debounceRef = useRef<null | NodeJS.Timeout>(null);

  const dataLessonThumbnail = useSelector(lessonThumbnail);

  const { data, error, isLoading, refetch } = useGetLessonSettingsQuery(
    lessonId as string,
    {
      refetchOnMountOrArgChange: true,
    }
  );

  const [updateLessonPreviewTitle] = useUpdateLessonPreviewTitleMutation();

  useEffect(() => {
    if (!data) return;

    dispatch(setPageEditorLessonState(data.State));
    dispatch(setLessonThumbnailTitle(data.Title));
    dispatch(setLessonThumbnailUrl(data.ThumbnailUrl));
  }, [data]);

  useEffect(() => {
    addWebSocketReconnectListener(refetch);

    return () => removeWebSocketReconnectListener(refetch);
  }, []);

  if (error) return <MyErrorResult />;

  return (
    <LessonPreviewCard
      mode="editable"
      lessonId={lessonId as string}
      loading={isLoading}
      lessonSettings={{
        Title: dataLessonThumbnail.title || "",
        ThumbnailUrl: dataLessonThumbnail.url || "",
      }}
      onEditTitle={(newTitle) => {
        try {
          if (debounceRef.current) {
            clearTimeout(debounceRef.current);
          }

          debounceRef.current = setTimeout(() => {
            try {
              updateLessonPreviewTitle({
                lessonId: lessonId as string,
                newTitle: newTitle,
              });

              dispatch(setLessonThumbnailTitle(newTitle));
            } catch (err) {
              console.error(err);
            }
          }, 3000);
        } catch (err) {
          console.error(err);
        }
      }}
    />
  );
};

const LessonContentComponents: React.FC = () => {
  const { lessonId } = useParams();
  const dispatch = useDispatch();

  const { data, error, isLoading, refetch } = useGetLessonContentsQuery(
    lessonId as string,
    {
      refetchOnMountOrArgChange: true,
    }
  );

  const lnContents = useSelector(lessonContents);

  useEffect(() => {
    if (!data) return;

    dispatch(setLessonContents(data));
  }, [data]);

  useEffect(() => {
    addWebSocketReconnectListener(refetch);

    return () => removeWebSocketReconnectListener(refetch);
  }, []);

  if (error) return <MyErrorResult />;

  return (
    <Card loading={isLoading}>
      <Flex vertical gap={16}>
        {!lnContents || lnContents.length == 0 ? (
          <MyEmpty />
        ) : (
          <Droppable items={lnContents} />
        )}
      </Flex>
    </Card>
  );
};

export default function LessonPageEditor() {
  const { t } = useTranslation();
  const { lessonId } = useParams();
  const navigate = useNavigate();

  const dispatch = useDispatch();

  useEffect(() => {
    dispatch(setEditorActive(true));
    dispatch(setCurrentLessonId(lessonId as string));

    return () => {
      dispatch(setEditorActive(false));
    };
  }, []);

  return (
    <>
      <HeaderBar
        theme="light"
        backTo={Constants.ROUTE_PATHS.LESSIONS.PAGE.replace(
          ":lessonId",
          lessonId as string
        )}
        onView={() =>
          navigate(
            Constants.ROUTE_PATHS.LESSIONS.PAGE.replace(
              ":lessonId",
              lessonId as string
            )
          )
        }
        sticky
      />

      <Flex justify="center" style={{ padding: 12 }}>
        <Flex justify="center" vertical className={styles.cardContainer}>
          <Typography.Paragraph type="secondary" style={{ margin: 0 }}>
            {t("lessonPageEditor.previewParagraph")}
          </Typography.Paragraph>
          <PreviewCard />

          <Typography.Paragraph
            type="secondary"
            style={{ margin: 0, paddingTop: 16 }}
          >
            {t("lessonPageEditor.pageContentParagraph")}
          </Typography.Paragraph>
          <LessonContentComponents />
        </Flex>
      </Flex>
    </>
  );
}
