import { LessonContent } from "core/types/lesson";
import { getTypeByName } from "./components";
import { Button, Input, Typography, Flex } from "antd";
import { useUpdateLessonContentMutation } from "core/services/lessons";
import { useSelector } from "react-redux";
import { currentLessonId } from "./LessonPageEditor/lessonPageEditorSlice";
import { useRef } from "react";
import MyUpload from "shared/components/MyUpload";
import { Constants } from "core/utils/utils";
import { MediaPlayer, MediaProvider } from "@vidstack/react";
import {
  defaultLayoutIcons,
  DefaultVideoLayout,
} from "@vidstack/react/player/layouts/default";
import "@vidstack/react/player/styles/default/theme.css";
import "@vidstack/react/player/styles/default/layouts/video.css";
import { darkMode } from "core/reducers/appSlice";
import { useTranslation } from "react-i18next";

const extractVideoId = (url: string) => {
  // regex to extract video id from youtube url
  const regex =
    /(?:https?:\/\/)?(?:www\.)?youtube\.com\/(?:watch\?v=|embed\/|v\/|.+\?v=|.+\/|.+v=)?([^"&?\/\s]{11})/;
  const match = url.match(regex);
  return match ? match[1] : url;
};

export function Converter({
  mode,
  lessonContent,
  onEdit,
}: {
  mode: "view" | "edititable";
  lessonContent: LessonContent;
  onEdit?: (newData: string) => void;
}) {
  const { t } = useTranslation();
  const debounceRef = useRef<null | NodeJS.Timeout>(null);

  const lessonId = useSelector(currentLessonId);
  const isDarkMode = useSelector(darkMode);

  const [reqUpdateLessonContent] = useUpdateLessonContentMutation();

  switch (lessonContent.Type) {
    case getTypeByName("Header"):
      if (mode === "view") {
        return (
          <div
            style={{ fontWeight: "bold", fontSize: 24, wordBreak: "break-all" }}
          >
            {lessonContent.Data}
          </div>
        );
      }

      return (
        <Input
          variant="borderless"
          style={{
            fontSize: 24,
            fontWeight: "bold",
            wordBreak: "break-all",
            padding: 0,
            margin: 0,
          }}
          placeholder={t("lessonComponentsConverter.titlePlaceholder")}
          value={lessonContent.Data}
          onChange={(event) => {
            onEdit?.(event.target.value);

            if (debounceRef.current) {
              clearTimeout(debounceRef.current);
            }

            debounceRef.current = setTimeout(() => {
              try {
                reqUpdateLessonContent({
                  lessonId: lessonId,
                  contentId: lessonContent.Id,
                  data: event.target.value,
                });
              } catch (err) {
                console.error(err);
              }
            }, 1000);
          }}
        />
      );
    case getTypeByName("Text"):
      if (mode === "view") {
        const formattedText = lessonContent.Data.split("\n").map(
          (line, index) => <li key={index}>{line || "\u00A0"}</li>
        );

        return (
          <ul
            style={{
              fontSize: 16,
              wordBreak: "break-all",
              padding: 0,
              margin: 0,
              listStyleType: "none",
            }}
          >
            {formattedText}
          </ul>
        );
      }

      return (
        <Input.TextArea
          autoSize
          variant="borderless"
          placeholder={t("lessonComponentsConverter.textPlaceholder")}
          style={{ width: "100%", padding: 0, paddingTop: 4 }}
          value={lessonContent.Data}
          onChange={(event) => {
            onEdit?.(event.target.value);

            if (debounceRef.current) {
              clearTimeout(debounceRef.current);
            }

            debounceRef.current = setTimeout(() => {
              try {
                reqUpdateLessonContent({
                  lessonId: lessonId,
                  contentId: lessonContent.Id,
                  data: event.target.value,
                });
              } catch (err) {
                console.error(err);
              }
            }, 1000);
          }}
        />
      );
    case getTypeByName("Image"):
      if (mode === "view" && lessonContent.Data === "") {
        return (
          <div
            style={{
              position: "relative",
              height: 120,
              width: "100%",
              backgroundColor: isDarkMode ? "#222" : "#EBEBEB",
              marginTop: 4,
              borderRadius: 4,
              marginRight: 8,
            }}
          >
            <div
              style={{
                position: "absolute",
                top: "50%",
                left: "50%",
                transform: "translate(-50%, -50%)",
              }}
            >
              {t("lessonComponentsConverter.noImageProvided")}
            </div>
          </div>
        );
      }

      const GalleryUpload = () => {
        return (
          <MyUpload
            action={`/lessons/${lessonId}/contents/${lessonContent.Id}/file/image`}
            onChange={(info) => {
              if (info.file.status === "done") {
                onEdit?.(info.file.response.Data);
              }
            }}
            imgCropProps={{
              aspect: 5 / 4,
              children: <></>,
            }}
          >
            <Button type="link">
              {t("lessonComponentsConverter.gallery")}
            </Button>
          </MyUpload>
        );
      };

      if (lessonContent.Data === "") {
        return (
          <div
            style={{
              position: "relative",
              height: 120,
              width: "100%",
              backgroundColor: isDarkMode ? "#222" : "#EBEBEB",
              borderRadius: 4,
              margin: "12px 12px 12px 0",
            }}
          >
            <div
              style={{
                display: "flex",
                flexDirection: "column",
                alignItems: "center",
                position: "absolute",
                top: "50%",
                left: "50%",
                transform: "translate(-50%, -50%)",
              }}
            >
              <span>{t("lessonComponentsConverter.chooseImageFrom")}</span>

              <GalleryUpload />
            </div>
          </div>
        );
      }

      return (
        <Flex vertical style={{ width: "100%" }}>
          <img
            src={`${Constants.STATIC_CONTENT_ADDRESS}/${lessonContent.Data}`}
            alt="img"
            style={{ width: "100%" }}
          />

          {mode === "edititable" && (
            <Flex
              style={{
                width: "100%",
                backgroundColor: isDarkMode ? "#222" : "#EBEBEB",
                marginTop: 4,
                borderRadius: 4,
              }}
              justify="center"
            >
              <div>
                <span>
                  {t("lessonComponentsConverter.chooseAnotherImageFrom")}
                </span>
                <GalleryUpload />
              </div>
            </Flex>
          )}
        </Flex>
      );
    case getTypeByName("YouTube"):
      const videoId = extractVideoId(lessonContent.Data);

      return (
        <Flex vertical style={{ width: "100%", paddingBottom: 12 }}>
          <iframe
            width="100%"
            height={mode === "view" ? 422 : 390}
            style={{ border: 0 }}
            src={`https://www.youtube.com/embed/${videoId}`}
            allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture; web-share"
          ></iframe>

          {mode === "edititable" && (
            <>
              <Typography.Text>
                {t("lessonComponentsConverter.videoId")}
              </Typography.Text>
              <Input
                placeholder="https://www.youtube.com/watch?v=Robzc9p1l50 or Robzc9p1l50"
                value={lessonContent.Data}
                onChange={(event) => {
                  onEdit?.(event.target.value);

                  if (debounceRef.current) clearTimeout(debounceRef.current);

                  if (event.target.value === "") return;

                  debounceRef.current = setTimeout(() => {
                    try {
                      reqUpdateLessonContent({
                        lessonId: lessonId,
                        contentId: lessonContent.Id,
                        data: extractVideoId(event.target.value),
                      });
                    } catch (err) {
                      console.error(err);
                    }
                  }, 1000);
                }}
              />
            </>
          )}
        </Flex>
      );
    case getTypeByName("Video"):
      if (mode === "view" && lessonContent.Data === "") {
        return (
          <div
            style={{
              position: "relative",
              height: 120,
              width: "100%",
              backgroundColor: isDarkMode ? "#222" : "#EBEBEB",
              marginTop: 4,
              borderRadius: 4,
              marginRight: 8,
            }}
          >
            <div
              style={{
                position: "absolute",
                top: "50%",
                left: "50%",
                transform: "translate(-50%, -50%)",
              }}
            >
              {t("lessonComponentsConverter.noVideoProvided")}
            </div>
          </div>
        );
      }

      const VideoUpload = () => {
        return (
          <MyUpload
            action={`/lessons/${lessonId}/contents/${lessonContent.Id}/file/video`}
            onChange={(info) => {
              if (info.file.status === "done") {
                onEdit?.(info.file.response.Data);
              }
            }}
            accept={Constants.ACCEPTED_VIDEO_FILE_TYPES}
            fileType="video"
          >
            <Button type="link">{t("lessonComponentsConverter.video")}</Button>
          </MyUpload>
        );
      };

      if (lessonContent.Data === "") {
        return (
          <div
            style={{
              position: "relative",
              height: 120,
              width: "100%",
              backgroundColor: isDarkMode ? "#222" : "#EBEBEB",
              borderRadius: 4,
              margin: "12px 12px 12px 0",
            }}
          >
            <div
              style={{
                display: "flex",
                flexDirection: "column",
                alignItems: "center",
                position: "absolute",
                top: "50%",
                left: "50%",
                transform: "translate(-50%, -50%)",
              }}
            >
              <span>{t("lessonComponentsConverter.chooseVideoFrom")}</span>

              <VideoUpload />
            </div>
          </div>
        );
      }

      return (
        <Flex vertical style={{ width: "100%" }}>
          <MediaPlayer
            load="idle"
            src={`${Constants.STATIC_CONTENT_ADDRESS}/${lessonContent.Data}`}
          >
            <MediaProvider />
            <DefaultVideoLayout icons={defaultLayoutIcons} />
          </MediaPlayer>

          {mode === "edititable" && (
            <Flex
              style={{
                width: "100%",
                backgroundColor: isDarkMode ? "#222" : "#EBEBEB",
                margin: "4px 0",
                borderRadius: 4,
                height: 48,
              }}
              justify="center"
              align="center"
            >
              <div>
                <span>
                  {t("lessonComponentsConverter.chooseAnotherVideoFrom")}
                </span>
                <VideoUpload />
              </div>
            </Flex>
          )}
        </Flex>
      );

    case getTypeByName("Iframe"):
      return (
        <div style={{ fontSize: 14, width: "100%" }}>
          {t("lessonComponentsConverter.notImplemented")}
        </div>
      );
    case getTypeByName("Banner"):
      return (
        <div style={{ fontSize: 14, width: "100%" }}>
          {t("lessonComponentsConverter.notImplemented")}
        </div>
      );

    default:
      return <div>{t("lessonComponentsConverter.unkownType")}</div>;
  }
}
