import { Button, Divider, Flex } from "antd";
import MyBanner from "shared/components/MyBanner";
import { MyContainer } from "shared/components/MyContainer";
import { PlusOutlined } from "@ant-design/icons";
import Search, { SearchProps } from "antd/es/input/Search";
import { useNavigate } from "react-router-dom";
import HeaderBar from "core/components/Header";
import {
  useCreateLessonMutation,
  useGetLessonsQuery,
} from "core/services/lessons";
import { Constants } from "core/utils/utils";
import MyEmpty from "shared/components/MyEmpty";
import MyErrorResult from "shared/components/MyResult";
import LessonPreviewCard from "shared/components/MyLessonPreviewCard";
import { useMessage } from "core/context/MessageContext";
import { useEffect } from "react";
import { useDispatch, useSelector } from "react-redux";
import {
  lessons,
  searchFilter,
  setLessons,
  setSearchFilter,
} from "./lessonsSlice";
import {
  addWebSocketReconnectListener,
  removeWebSocketReconnectListener,
} from "core/services/websocketService";
import MyCenteredSpin from "shared/components/MyCenteredSpin";
import { useTranslation } from "react-i18next";

const CreateLessonButton: React.FC = () => {
  const navigate = useNavigate();
  const { t } = useTranslation();

  const [createLesson, { isLoading }] = useCreateLessonMutation();
  const { success, error } = useMessage();

  const handleCreateLesson = async () => {
    try {
      const res = await createLesson({}).unwrap();

      if (res && res.Id) {
        success(t("lessons.messageLessonSuccessfullyCreated"));

        navigate(
          Constants.ROUTE_PATHS.LESSIONS.PAGE_EDITOR.replace(
            ":lessonId",
            res.Id
          )
        );
      }
    } catch (err) {
      console.error(err);
      error(t("common.messageRequestFailed"));
    }
  };

  return (
    <Button
      icon={<PlusOutlined />}
      onClick={handleCreateLesson}
      loading={isLoading}
    >
      {t("common.create")}
    </Button>
  );
};

const LessonList: React.FC = () => {
  const { t } = useTranslation();
  const dispatch = useDispatch();
  const dataLessons = useSelector(lessons);
  const dataSearchFilter = useSelector(searchFilter);

  const { data, error, isLoading, refetch } = useGetLessonsQuery(undefined, {
    refetchOnMountOrArgChange: true,
  });

  useEffect(() => {
    if (!data) return;

    dispatch(setLessons(data));
  }, [data]);

  useEffect(() => {
    addWebSocketReconnectListener(refetch);

    return () => removeWebSocketReconnectListener(refetch);
  }, []);

  if (isLoading) return <MyCenteredSpin height="240px" />;
  if (error) return <MyErrorResult />;

  if (!dataLessons || dataLessons.length === 0) return <MyEmpty />;

  const filteredItems = dataLessons.filter((item) =>
    item.Title.toLowerCase().includes(dataSearchFilter.toLowerCase())
  );

  const publishedItems = filteredItems.filter((item) => item.State === 1);
  const unpublishedItems = filteredItems.filter((item) => item.State === 2);

  return (
    <>
      {publishedItems.length > 0 && (
        <>
          {publishedItems.map((item, index) => (
            <LessonPreviewCard
              key={index}
              mode="view"
              lessonId={item.Id}
              loading={false}
              lessonSettings={{
                Title: item.Title,
                ThumbnailUrl: item.ThumbnailUrl,
                QuestionsCount: item.QuestionsCount,
              }}
            />
          ))}
        </>
      )}

      {unpublishedItems.length > 0 && (
        <>
          <Divider orientation="left" style={{ marginBottom: 0 }}>
            {t("lessons.lessonStatusDrafts")}
          </Divider>

          {unpublishedItems.map((item, index) => (
            <LessonPreviewCard
              key={index}
              mode="view"
              lessonId={item.Id}
              loading={false}
              lessonSettings={{
                Title: item.Title,
                ThumbnailUrl: item.ThumbnailUrl,
                QuestionsCount: item.QuestionsCount,
              }}
            />
          ))}
        </>
      )}
    </>
  );
};

export default function Lessons() {
  const { t } = useTranslation();

  const dispatch = useDispatch();

  useEffect(() => {
    return () => {
      dispatch(setSearchFilter(""));
    };
  }, []);

  return (
    <>
      <MyBanner title={t("lessons.bannerTitle")} headerBar={<HeaderBar />} />

      <MyContainer>
        <Flex justify="right" gap={16} style={{ paddingBottom: 16 }}>
          <CreateLessonButton />

          <Search
            placeholder={t("lessons.searchPlaceholder")}
            onChange={(e) => {
              dispatch(setSearchFilter(e.target.value));
            }}
            style={{ width: 300 }}
            allowClear
          />
        </Flex>

        <Flex vertical gap={16}>
          <LessonList />
        </Flex>
      </MyContainer>
    </>
  );
}
