import { Avatar, Checkbox, Collapse, Form, Tooltip } from "antd";
import HeaderBar from "core/components/Header";
import MyBanner from "shared/components/MyBanner";
import { MyContainer } from "shared/components/MyContainer";
import MyErrorResult from "shared/components/MyResult";
import MyEmpty from "shared/components/MyEmpty";
import MyCenteredSpin from "shared/components/MyCenteredSpin";
import { Role } from "core/types/organization";
import { useGetRolesQuery } from "core/services/organization";
import { useEffect } from "react";
import {
  addWebSocketReconnectListener,
  removeWebSocketReconnectListener,
} from "core/services/websocketService";
import MyUserAvatar from "shared/components/MyUserAvatar";
import { useTranslation } from "react-i18next";

export default function Roles() {
  const { t } = useTranslation();

  const { data, error, isLoading, refetch } = useGetRolesQuery(undefined, {
    refetchOnMountOrArgChange: true,
  });

  useEffect(() => {
    addWebSocketReconnectListener(refetch);

    return () => removeWebSocketReconnectListener(refetch);
  }, []);

  return (
    <>
      <MyBanner
        title={t("roles.bannerTitle")}
        subtitle={t("common.bannerSubtitle")}
        headerBar={<HeaderBar />}
      />

      <MyContainer
        style={{
          display: "flex",
          flexDirection: "column",
          gap: 16,
        }}
      >
        {error ? (
          <MyErrorResult />
        ) : isLoading ? (
          <MyCenteredSpin height="240px" />
        ) : data === undefined ? (
          <MyEmpty />
        ) : (
          <>
            {data.Roles.map((role, index) => (
              <RoleComponent key={index} role={role} />
            ))}
          </>
        )}
      </MyContainer>
    </>
  );
}

interface Permission {
  id: number;
  category: string;
  title: string;
  description: string;
}

// test data

/*
const tmpI18nObj = [
  {
    id: 1,
    category: "Team",
    title: "Invite new team member",
    description:
      "Permission to invite a member. An email will be sent to the new member.",
  },
  {
    id: 2,
    category: "Team",
    title: "Remove team member",
    description: "Permission to remove a member.",
  },
  {
    id: 3,
    category: "Roles",
    title: "Create new role",
    description:
      "Permission to invite a member. An email will be sent to the new member.",
  },
  {
    id: 4,
    category: "Roles",
    title: "Delete role",
    description:
      "Permission to invite a member. An email will be sent to the new member.",
  },
]; */

/*
export const tmpRoleNames = {
  "d0f0fa0d-3f3b-438b-a76f-7febeb8aab57": "Admin",
  "b7359e12-359e-423b-b39c-f0d4069adebc": "Moderator",
  "a1f084ad-d501-4015-b326-4c5c46fd1c5e": "User",
} as any; */

function RoleComponent({ role }: { role: Role }) {
  const { t } = useTranslation();

  const tmpI18nRoles = t("roles.roles", {
    returnObjects: true,
  }) as Permission[];

  const tmpRoleNames = t("roles.roleNames", {
    returnObjects: true,
  }) as any;

  const teamPermissions = tmpI18nRoles.filter(
    (permission) => permission.category === "Team"
  );
  const rolePermissions = tmpI18nRoles.filter(
    (permission) => permission.category === "Roles"
  );

  const MyCheckbox = ({ permission }: { permission: Permission }) => {
    return (
      <Form.Item extra={permission.description}>
        <Checkbox disabled checked={role.Permissions.includes(permission.id)}>
          {permission.title}
        </Checkbox>
      </Form.Item>
    );
  };

  return (
    <Form>
      <Collapse
        defaultActiveKey={["1"]}
        items={[
          {
            key: "1",
            label: tmpRoleNames[role.Id],
            children: (
              <Collapse
                defaultActiveKey={["1"]}
                ghost
                items={[
                  {
                    key: "1",
                    label: t("roles.collapseTitleTeam"),
                    children: (
                      <>
                        {teamPermissions.map((permission, index) => (
                          <MyCheckbox key={index} permission={permission} />
                        ))}
                      </>
                    ),
                  },
                  {
                    key: "2",
                    label: t("roles.collapseTitleRoles"),
                    children: (
                      <>
                        {rolePermissions.map((permission, index) => (
                          <MyCheckbox key={index} permission={permission} />
                        ))}
                      </>
                    ),
                  },
                ]}
              />
            ),
            extra: (
              <div
                onClick={(event) => {
                  event.stopPropagation();
                }}
              >
                {role.Users.length > 0 && (
                  <Avatar.Group
                    size="small"
                    max={{
                      count: 4,
                    }}
                  >
                    {role.Users.map((user, index) => (
                      <Tooltip
                        key={index}
                        title={`${user.FirstName} ${user.LastName}`}
                        placement="top"
                      >
                        <div>
                          <MyUserAvatar
                            profilePictureUrl={user.ProfilePictureUrl}
                            firstName={user.FirstName}
                            size={24}
                          />
                        </div>
                      </Tooltip>
                    ))}
                  </Avatar.Group>
                )}
              </div>
            ),
          },
        ]}
      />
    </Form>
  );
}

/*
function RoleComponent({ role }: { role: Role }) {
    const [expandIconPosition, setExpandIconPosition] = useState<ExpandIconPosition>('start');

    const [editMode, setEditMode] = useState(false);
    const [form] = useForm();

    const onPositionChange = (newExpandIconPosition: ExpandIconPosition) => {
        setExpandIconPosition(newExpandIconPosition);
    };

    const onChange = (key: string | string[]) => {};

    const teamPermissions = [
        {
            id: 1,
            label: 'Invite new team member',
            description: 'Permission to invite a member. An email will be sent to the new member.',
        },
        {
            id: 2,
            label: 'Remove team member',
            description: 'Permission to invite a member. An email will be sent to the new member.',
        },
    ];

    const rolePermissions = [
        {
            id: 3,
            label: 'Invite new team member',
            description: 'Permission to invite a member. An email will be sent to the new member.',
        },
        {
            id: 4,
            label: 'Invite new team member',
            description: 'Permission to invite a member. An email will be sent to the new member.',
        },
    ];

    const MyCheckbox = ({ name, label, description }: { name: number; label: string; description: string }) => {
        return (
            <FormItem name={name} valuePropName="checked" extra={description}>
                <Checkbox disabled={role.Master || (!role.Master && !editMode)}>{label}</Checkbox>
            </FormItem>
        );
    };

    const handleSave = () => {
        console.log(form.getFieldsValue());
    };

    /*
    useEffect(() => {
        // set all checkboxes to true if role is master
        if (role.Master) {
            const obj = {} as any;
            const objPermissionValues = Object.values(Constants.PERMISSIONS);

            for (let i = 1; i <= Object.keys(Constants.PERMISSIONS).length; i++) {
                obj[objPermissionValues[i - 1]] = true;
            }

            form.setFieldsValue(obj);
        }
    }, []);
    */
/*

    return (
        <Form form={form}>
            <Collapse
                defaultActiveKey={['1']}
                collapsible={editMode ? 'icon' : 'header'}
                onChange={onChange}
                expandIconPosition={expandIconPosition}
                items={[
                    {
                        key: '1',
                        label: editMode ? <Input placeholder="Role name" value={'Admin'} size="small" /> : role.Name,
                        children: (
                            <>
                                {role.Master && (
                                    <Typography.Text type="secondary" italic>
                                        Permissions for this role cannot be changed as it the master role.
                                    </Typography.Text>
                                )}

                                <Collapse
                                    defaultActiveKey={['1']}
                                    ghost
                                    onChange={onChange}
                                    expandIconPosition={expandIconPosition}
                                    items={[
                                        {
                                            key: '1',
                                            label: 'Team',
                                            children: (
                                                <>
                                                    {teamPermissions.map((permission) => (
                                                        <MyCheckbox key={permission.id} name={permission.id} label={permission.label} description={permission.description} />
                                                    ))}
                                                </>
                                            ),
                                        },
                                        {
                                            key: '2',
                                            label: 'Roles',
                                            children: (
                                                <>
                                                    {rolePermissions.map((permission) => (
                                                        <MyCheckbox key={permission.id} name={permission.id} label={permission.label} description={permission.description} />
                                                    ))}
                                                </>
                                            ),
                                        },
                                    ]}
                                />
                            </>
                        ),
                        extra: (
                            <Space style={{ paddingLeft: 12 }}>
                                {!editMode ? (
                                    <>
                                        {role.Users.length > 0 && (
                                            <Avatar.Group
                                                size="small"
                                                max={{
                                                    count: 2,
                                                }}
                                            >
                                                {role.Users.map((user) => (
                                                    <Tooltip title={`${user.FirstName} ${user.LastName}`} placement="top">
                                                        <Avatar style={{ backgroundColor: '#f56a00' }}>{user.FirstName[0]}</Avatar>
                                                    </Tooltip>
                                                ))}
                                            </Avatar.Group>
                                        )}
                                    </>
                                ) : (
                                    <>
                                        <DeleteOutlined />
                                        <SaveOutlined
                                            onClick={(event) => {
                                                event.stopPropagation();

                                                setEditMode(false);

                                                handleSave();
                                            }}
                                        />
                                        <CloseOutlined
                                            onClick={(event) => {
                                                event.stopPropagation();

                                                setEditMode(false);
                                            }}
                                        />
                                    </>
                                )}
                            </Space>
                        ),
                    },
                ]}
            />
        </Form>
    );
}
*/

/*
const CreateRoleButton: React.FC = () => {
    const [createRole, { isLoading }] = useCreateRoleMutation();

    const handleCreateRole = async () => {
        try {
            createRole('New Role');
        } catch (err) {
            console.error(err);
        }
    };

    return (
        <Button icon={<PlusOutlined />} onClick={handleCreateRole} loading={isLoading}>
            Create role
        </Button>
    );
};
*/
