import MyTable from "shared/components/MyTable";
import HeaderBar from "core/components/Header";
import MyBanner from "shared/components/MyBanner";
import { MyContainer } from "shared/components/MyContainer";
import { Badge, Button, Flex, Popconfirm, Select, Space } from "antd";
import { UserAddOutlined } from "@ant-design/icons";
import { Link } from "react-router-dom";
import { Constants } from "core/utils/utils";
import {
  useDeleteTeamMemberMutation,
  useGetTeamQuery,
  useUpdateTeamMemberRoleMutation,
} from "core/services/organization";
import MyErrorResult from "shared/components/MyResult";
import { useEffect, useState } from "react";
import { useDispatch, useSelector } from "react-redux";
import { setTeamMembers, teamMembers } from "./teamSlice";
import {
  addWebSocketReconnectListener,
  removeWebSocketReconnectListener,
} from "core/services/websocketService";
import { useMessage } from "core/context/MessageContext";
import MyUserAvatar from "shared/components/MyUserAvatar";
import { useTranslation } from "react-i18next";

const TeamList: React.FC = () => {
  const { t } = useTranslation();
  const dispatch = useDispatch();
  const { success, error: errorMessage } = useMessage();

  const dataTeamMembers = useSelector(teamMembers);

  const [selectedRoleId, setSelectedRoleId] = useState<string | undefined>();

  const { data, error, isLoading, refetch } = useGetTeamQuery(undefined, {
    refetchOnMountOrArgChange: true,
  });

  const [reqUpdateTeamMemberRole, { isLoading: loadingUpdateTeamMemberRole }] =
    useUpdateTeamMemberRoleMutation();

  const [reqDeleteTeamMember, { isLoading: loadingDeleteTeamMember }] =
    useDeleteTeamMemberMutation();

  const tmpRoleNames = t("roles.roleNames", {
    returnObjects: true,
  }) as any;

  const getTableContent = () => {
    let items = [
      {
        title: t("common.firstName"),
        dataIndex: "firstName",
        key: "firstName",
      },
      {
        title: t("common.lastName"),
        dataIndex: "lastName",
        key: "lastName",
      },
      {
        title: t("common.email"),
        dataIndex: "email",
        key: "email",
      },
      {
        title: t("common.role"),
        dataIndex: "role",
        key: "role",
      },
      {
        title: t("common.status"),
        dataIndex: "status",
        key: "status",
      },
      {
        title: t("common.actions"),
        dataIndex: "actions",
        key: "actions",
        render: (_: any, record: any) => (
          <Space size="middle">
            <Popconfirm
              title={t("team.popConfirmRoleChange.title")}
              onConfirm={async () => {
                try {
                  await reqUpdateTeamMemberRole({
                    memberId: record.key,
                    roleId: selectedRoleId,
                  }).unwrap();

                  success(
                    t(
                      "team.popConfirmRoleChange.messageUpdatedRoleSuccessfully"
                    )
                  );
                } catch (error) {
                  console.error(error);
                  errorMessage(t("common.messageRequestFailed"));
                }
              }}
              okButtonProps={{ loading: loadingUpdateTeamMemberRole }}
              cancelText={t("common.cancel")}
              description={
                <Select
                  style={{ width: 150 }}
                  defaultValue={record.role}
                  value={selectedRoleId}
                  onChange={(value) => setSelectedRoleId(value)}
                >
                  {Object.keys(tmpRoleNames).map((key) => (
                    <Select.Option key={key} value={key}>
                      {tmpRoleNames[key]}
                    </Select.Option>
                  ))}
                </Select>
              }
            >
              <Button
                type="link"
                onClick={() => setSelectedRoleId(record.role)}
              >
                {t("team.changeRole")}
              </Button>
            </Popconfirm>

            <Popconfirm
              title={t("team.popConfirmDeleteMember.title")}
              okButtonProps={{
                loading: loadingDeleteTeamMember,
              }}
              cancelText={t("common.cancel")}
              onConfirm={async () => {
                try {
                  await reqDeleteTeamMember(record.key).unwrap();

                  success(
                    t(
                      "team.popConfirmDeleteMember.messageDeletedMemberSuccessfully"
                    )
                  );
                } catch (error) {
                  console.error(error);
                  errorMessage(t("common.messageRequestFailed"));
                }
              }}
            >
              <Button type="link">{t("common.delete")}</Button>
            </Popconfirm>
          </Space>
        ),
      },
    ];

    return items;
  };

  const getTableItems = () => {
    let items = [] as any[];

    if (!dataTeamMembers) return items;

    dataTeamMembers.forEach((item) => {
      items.push({
        key: item.Id,
        firstName: (
          <Space>
            <MyUserAvatar
              size={42}
              profilePictureUrl={item.ProfilePictureUrl}
              firstName={item.FirstName}
              disableCursorPointer
            />
            <span>{item.FirstName}</span>
          </Space>
        ),
        lastName: item.LastName,
        email: item.Email,
        role: tmpRoleNames[item.RoleId],
        status: (
          <Badge
            status={item.Online ? "success" : "error"}
            text={item.Online ? "Online" : "Offline"}
          />
        ),
      });
    });

    items.sort((a, b) => a.role.localeCompare(b.role));

    return items;
  };

  useEffect(() => {
    if (!data) return;

    dispatch(setTeamMembers(data));
  }, [data]);

  useEffect(() => {
    addWebSocketReconnectListener(refetch);

    return () => removeWebSocketReconnectListener(refetch);
  }, []);

  if (error) return <MyErrorResult />;

  return (
    <MyTable
      loading={isLoading}
      columns={getTableContent()}
      dataSource={getTableItems()}
    />
  );
};

export default function Team() {
  const { t } = useTranslation();

  return (
    <>
      <MyBanner
        title={t("team.bannerTitle")}
        subtitle={t("common.bannerSubtitle")}
        headerBar={<HeaderBar />}
      />

      <MyContainer
        style={{
          display: "flex",
          flexDirection: "column",
          gap: 16,
        }}
      >
        <Flex justify="end">
          <Link to={Constants.ROUTE_PATHS.ORGANIZATION_TEAM_CREATE_USER}>
            <Button icon={<UserAddOutlined />}>
              {t("team.addMemberButton")}
            </Button>
          </Link>
        </Flex>

        <TeamList />
      </MyContainer>
    </>
  );
}
