import {
  Card,
  Descriptions,
  Flex,
  Form,
  Input,
  Select,
  Typography,
} from "antd";
import HeaderBar from "core/components/Header";
import MyBanner from "shared/components/MyBanner";
import MyMiddleCard from "shared/components/MyMiddleCard";
import Meta from "antd/es/card/Meta";
import { useGetUserProfileQuery } from "core/services/userProfile";
import { useEffect } from "react";
import {
  addWebSocketReconnectListener,
  removeWebSocketReconnectListener,
} from "core/services/websocketService";
import { useDispatch, useSelector } from "react-redux";
import {
  email,
  firstName,
  lastName,
  profilePictureUrl,
  roleId,
  setEmail,
  setFirstName,
  setLastName,
  setProfilePictureUrl,
  setRoleId,
} from "./userProfileSlice";
import MyErrorResult from "shared/components/MyResult";
import MyUpload from "shared/components/MyUpload";
import { useMessage } from "core/context/MessageContext";
import MyUserAvatar from "shared/components/MyUserAvatar";
import { setUserProfilePictureUrl } from "core/reducers/appSlice";
import { useTranslation } from "react-i18next";

export default function UserProfile({ isAdmin }: { isAdmin?: boolean }) {
  const dispatch = useDispatch();
  const { t, i18n } = useTranslation();
  const { success } = useMessage();

  const dataProfilePictureUrl = useSelector(profilePictureUrl);
  const dataFirstName = useSelector(firstName);
  const dataLastName = useSelector(lastName);
  const dataEmail = useSelector(email);
  const dataRoleId = useSelector(roleId);

  const { data, error, isLoading, refetch } = useGetUserProfileQuery(
    undefined,
    {
      refetchOnMountOrArgChange: true,
    }
  );

  const tmpRoleNames = t("roles.roleNames", {
    returnObjects: true,
  }) as any;

  function AdminWrapper({ children }: { children: React.ReactNode }) {
    if (!isAdmin) {
      return <>{children}</>;
    }

    return (
      <Form layout="vertical" style={{ marginTop: 24 }}>
        {children}
      </Form>
    );
  }

  function TextItem({ value, name }: { value: string; name: string }) {
    if (!isAdmin) {
      return <>{value}</>;
    }

    return (
      <Form.Item name={name} style={{ width: "100%" }} required>
        <Input defaultValue={value} />
      </Form.Item>
    );
  }

  useEffect(() => {
    if (!data) return;

    dispatch(setProfilePictureUrl(data.ProfilePictureUrl));
    dispatch(setFirstName(data.FirstName));
    dispatch(setLastName(data.LastName));
    dispatch(setEmail(data.Email));
    dispatch(setRoleId(data.RoleId));
  }, [data]);

  useEffect(() => {
    addWebSocketReconnectListener(refetch);

    return () => removeWebSocketReconnectListener(refetch);
  }, []);

  return (
    <>
      <MyBanner
        title={t("userProfile.bannerTitle")}
        subtitle={t("common.bannerSubtitle")}
        headerBar={<HeaderBar />}
      />

      <MyMiddleCard
        title={t("userProfile.middleCardTitle")}
        loading={isLoading}
      >
        {error ? (
          <MyErrorResult />
        ) : (
          <Flex vertical gap={16} style={{ paddingBottom: 16 }}>
            <Card
              styles={{
                body: {
                  padding: 16,
                },
              }}
            >
              <Form
                layout="vertical"
                initialValues={{
                  language: i18n.language,
                }}
              >
                <Flex justify="space-between" align="center" wrap gap={24}>
                  <Meta
                    avatar={
                      <MyUpload
                        action={`/user/profile/picture`}
                        onChange={(info) => {
                          if (info.file.status === "done") {
                            success(
                              t("userProfile.messageProfileSuccessfullyUpdated")
                            );

                            dispatch(
                              setProfilePictureUrl(info.file.response.Data)
                            );
                            dispatch(
                              setUserProfilePictureUrl(info.file.response.Data)
                            );
                          }
                        }}
                        imgCropProps={{
                          aspect: 1 / 1,
                          children: <></>,
                        }}
                      >
                        <MyUserAvatar
                          profilePictureUrl={dataProfilePictureUrl}
                        />
                      </MyUpload>
                    }
                    title={`${dataFirstName} ${dataLastName}`}
                    description={tmpRoleNames[dataRoleId]}
                  />

                  <Form.Item label={t("userProfile.language")} name="language">
                    <Select
                      onChange={(value) => i18n.changeLanguage(value)}
                      style={{ width: 120 }}
                    >
                      <Select.Option value="en">
                        {t("userProfile.languageEnglish")}
                      </Select.Option>
                      <Select.Option value="de">
                        {t("userProfile.languageGerman")}
                      </Select.Option>
                    </Select>
                  </Form.Item>
                </Flex>
              </Form>
            </Card>

            <Card
              styles={{
                body: {
                  padding: 16,
                },
              }}
            >
              <AdminWrapper>
                <Descriptions
                  title={t("userProfile.personalInformation")}
                  layout="vertical"
                  items={[
                    {
                      key: "1",
                      label: t("common.firstName"),
                      children: (
                        <TextItem value={dataFirstName} name="firstName" />
                      ),
                    },
                    {
                      key: "2",
                      label: t("common.lastName"),
                      children: (
                        <TextItem value={dataLastName} name="lastName" />
                      ),
                    },
                    {
                      key: "3",
                      label: t("common.email"),
                      children: <TextItem value={dataEmail} name="email" />,
                    },
                  ]}
                />
              </AdminWrapper>
            </Card>
          </Flex>
        )}
      </MyMiddleCard>
    </>
  );
}

/*
// TODO: sessions table

  <Card
    styles={{
      body: {
        padding: 16,
      },
    }}
  >
    <Meta title="Sessions" />
    <Typography.Text>// Todoo</Typography.Text>
  </Card>
*/

function TitleText({ children }: { children: React.ReactNode }) {
  return (
    <Typography.Text style={{ fontSize: 16, opacity: 0.4 }}>
      {children}
    </Typography.Text>
  );
}

function ValueText({ children }: { children: React.ReactNode }) {
  return <Typography.Text style={{ fontSize: 16 }}>{children}</Typography.Text>;
}
